// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function AdjustPlotBounds(RRBounds,VtBounds,IFBounds,IAmpBounds,TimeBounds,fig)
//Change the data bounds of the plots generated by CDM_SPWVD_Analysis
//RRBounds a vector [RRmin RRmax] or []
//VtBounds a vector [Vtmin Vtmax] or []
//IFBounds a vector [IFmin IFmax] or []
//IAmpBounds a vector [IAmpmin IAmpmax] or []
//TimeBounds a vector [Timemin Timemax] or []
//fig : the handle on the graphic window. If omitted the changes are
//      applied to the current figure  
//If a vector is set to [] the current bounds value is unchanged
  
  if argn(2)<6 then fig=gcf(),end
  errmsg=_("%s: Wrong size for input argument %d: An array of size %d expected.\n")
  A=fig.children;
  
  if IAmpBounds==[] then
    IAmpBounds=A(1).data_bounds(:,2)
  elseif size(IAmpBounds,'*')<>2 then 
    error(msprintf(errmsg,"AdjustPlotBounds",4,2))
  end
  if IFBounds==[] then
    IFBounds=A(2).data_bounds(:,2)
  elseif size(IFBounds,'*')<>2 then 
    error(msprintf(errmsg,"AdjustPlotBounds",3,2))
  end
  if size(A,'*')==4 then
    if VtBounds==[] then
      VtBounds=A(3).data_bounds(:,2)
    elseif size(VtBounds,'*')<>2 then 
      error(msprintf(errmsg,"AdjustPlotBounds",2,2))
    end
  end
  if RRBounds==[] then
    RRBounds=A($).data_bounds(:,2)
  elseif size(RRBounds,'*')<>2 then 
    error(msprintf(errmsg,"AdjustPlotBounds",1,2))
  end

  if TimeBounds==[] then
    TimeBounds=A(1).data_bounds(:,1)
  elseif size(TimeBounds,'*')<>2 then 
    error(msprintf(errmsg,"AdjustPlotBounds",4,2))
  end
  
  drawlater()
  A(1).data_bounds=[TimeBounds(:) IAmpBounds(:)]
  A(2).data_bounds=[TimeBounds(:) IFBounds(:)]
  if size(A,'*')==4 then
    A(3).data_bounds=[TimeBounds(:) VtBounds(:)]
  end
  A($).data_bounds=[TimeBounds(:) RRBounds(:)]

  drawnow()
endfunction
