// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function Ax=MultiChannelPlot(t,varargin)
  na=size(varargin)
  colors=[color('blue'),color('magenta'),color('black'),color('red'),color('orange'),color('green')];
  grid_color=color("gray");
  font_style=6;
  font_size=1;  
  Title=[];
  if typeof(varargin(na))=="st" then
    legalfields=["curve_colors"  "grid_color" "title" "font_style" "font_size"]
    options=varargin(na)

    fn=fieldnames(options)
    for k=1:size(fn,'*')
      select fn(k)
      case "curve_colors" then
        colors=options.curve_colors
      case "grid_color" then
        grid_color=options.grid_color
      case "title" then
        Title=options.title
      case "font_style" then
        font_style=options.font_style
      case "font_size" then
        font_size=options.font_size
      else
       error(msprintf(_("%s: Wrong value for argument %d: Struct fields must be in {%s}\n"),...
                      "MultiChannelPlot",na+1,strcat(legalfields,','))) 
      end
    end
    na=na-1
  end
  
  if modulo(na,2)<>0 then 
    error(msprintf(_("%s: Wrong number of arguments : Each curve must be described by a pair of arguments.\n"),...
                   "MultiChannelPlot"))
  end
  nplot=na/2;
  nt=size(t,'*');t=t(:);
  for k=1:2:na
    signal=varargin(k);
    label=varargin(k+1);
    if type(signal)<>1|~isreal(signal) then
      error(msprintf(_("%s: Wrong type for argument %d: Real matrix expected.\n"),...
                   "MultiChannelPlot",k+1))
    end
    if and(size(signal)>1) then 
      if size(signal,1)<>nt then
        error(msprintf(_("%s: Wrong size for argument %d: A %d column expected.\n"),...
                   "MultiChannelPlot",k+1,nt))
      end
    else
      if size(signal,'*')<>nt then
        error(msprintf(_("%s: Wrong size for input argument #%d: A %d elements array expected.\n"),...
                       "MultiChannelPlot",k+1,nt))
      end

      varargin(k)=signal(:)
    end
   
    
    if type(label)<>10|size(label,'*')<>1 then
      error(msprintf(_("%s: Wrong type for input argument #%d: A character string expected.\n"),...
                       "MultiChannelPlot",k+2))
    end

  end
  o=0.05; //reserve space for title
  h=(1-2*o)/nplot;
  low_margin=0.05;
  nc=size(colors,'*')
  clf;fig=gcf();
  idraw=fig.immediate_drawing;
  fig.immediate_drawing = "off";
  tbounds=[min(t);max(t)]
  Ax=[]
  for kplot=1:nplot
    signal=varargin(2*kplot-1);
    if kplot==1 then 
      a=gca(), 
      if Title<>[] then a.title.text=Title;end
    else  
      a=newaxes();
    end
    a.axes_bounds=[0,o+(kplot-1)*h,1,h];
    a.tight_limits="on";
    a.margins(4)=low_margin;
    a.grid(1:2)=grid_color;
    a.font_style=font_style
    a.font_size=font_size
    
    mny=min(signal)
    mxy=max(signal)
    dy=(mxy-mny)/10;
    a.data_bounds=[tbounds [mny-dy;mxy+dy]]
    a.y_label.text=varargin(2*kplot);
    a.axes_visible='on';
    a.clip_state="clipgrf";
    for i=1:size(signal,2)
      xpoly(t,signal(:,i));
      p=gce();
      p.foreground=colors(modulo(i-1,nc)+1);
    end
    if kplot<>nplot then
      a.x_ticks.labels=emptystr(a.x_ticks.labels)
    else
       a.x_label.text=_("time (s)")
    end
    Ax=[Ax a]
  end
  fig.event_handler="MultiChannelPlotHandler";
  fig.event_handler_enable='off';

  step=round((tbounds(2)-tbounds(1))/10);
  ud=fig.user_data;
  if typeof(ud)<>"st" then
    ud=struct();
  end
  ud.type="MultiChannelPlot";
  ud.colors=colors;
  ud.scroll=[0,step,tbounds']
  set(fig,"user_data",ud);
  submenus=[_("Set scroll length"); _("Scroll On");_("Scroll Off")];
  delmenu(fig.figure_id,_("&Scroll"))
  addmenu(fig.figure_id,_("&Scroll"),submenus,list(2,"MultiChannelScrollMenu"))
  fig.immediate_drawing = idraw;
  
endfunction


