// This file is part of the CardioVascular toolbox
// Copyright (C) - INRIA - Alessandro Monti, Claire Médigue, Michel Sorine
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function []=ResamplingBP(listfile_in,listfile_out,res_par)

  input_param=0;
  visualrec=1;
  
  //Input file name acquisition
  txt=[_("Systolic Blood Pressure file name");..
       _("Diastolic Blood Pressure file name")];

  title=["    "+_("Enter resampling parameters");..
         "    "+_("Reconstructed file names")];
  if ~((type(listfile_in)==10)&(size(listfile_in)==[2,1])) then
    input_param=1;
    SBPi_name='SBPvect';
    DBPi_name='DBPvect';
  else
    SBPi_name=listfile_in(1);
    DBPi_name=listfile_in(2);
  end
  error_f=1;
  while ((error_f==1)&(visualrec==1)),
    error_f=0;
    i_value=[SBPi_name;DBPi_name];
    if (input_param==1) then
      answer_m=x_mdialog(title,txt,i_value);
      if ~(answer_m==[]) then
        SBPi_name=answer_m(1);
        DBPi_name=answer_m(2);
      else
        visualrec=0;
      end
    end
    if (visualrec==1) then
      // test input file
      unix_res=unix("ls "+SBPi_name);
      if ~(unix_res==0) then
        x_message(msprintf(_(" ERROR: file %s does not exist"),"SBPi_name"))
        error_f=1;input_param=1;
      end
      unix_res=unix("ls "+DBPi_name);
      if ~(unix_res==0) then
        x_message(msprintf(_(" ERROR: file %s does not exist"),"DBPi_name"))
        error_f=1;input_param=1;
      end
    end
  end
  
  //Output filename acquisition
  if (visualrec==1) then
    //   RECONSTRUCTION FILES
    //
    input_param2=0;
    if ~((type(listfile_out)==10)&(size(listfile_out)==[2,1])) then
      input_param2=1;
      SBPo_name='SBPseries';
      DBPo_name='DBPseries';
    else
      SBPo_name=listfile_out(2);
      DBPo_name=listfile_out(3);
    end
    error_f=1;
    txt=[_("Systolic Blood Pressure file name");..
         _("Diastolic Blood Pressure file name")];
    title=_("Enter RESAMPLED  files names');
    while ((error_f==1)&(visualrec==1)),
      error_f=0;
      o_value=[SBPo_name;DBPo_name];
      if (input_param2==1) then
        answer_m=x_mdialog(title,txt,o_value);
        if ~(answer_m==[]) then
          SBPo_name=answer_m(1);
          DBPo_name=answer_m(2);
        else
          visualrec=0;
        end
      end
    end
  end
  input_param3=0;
  if (visualrec==1) then
    // read of  reconstructed files
    SBPvect=read(SBPi_name,-1,2);
    DBPvect=read(DBPi_name,-1,2);
    //   resampling PARAMETERS
    if ~((type(res_par)==1)&(size(res_par)==[3,1])) then
      input_param3=1;
      res=0.5;     
      initres=max(min(DBPvect(:,2)),min(SBPvect(:,2)));
      initres=(round(initres/res))*res+res;
      endres=min(max(DBPvect(:,2)),max(SBPvect(:,2)));     
      endres=(round(endres/res))*res-res;
    else
      initres=res_par(1);
      endres=res_par(2);
      res=res_par(3); // resampling step        
      initres=(round(initres/res))*res;
      endres=(round(endres/res))*res;
    end    
    error_f=1;
    txt=[_("Init resampling time");..
         _("End resampling time");..
         _("Resampling step")];
    title=['  '+_("Resampling parameters")];
    while ((error_f==1)&(visualrec==1)),
      error_f=0;
      vis_value=[string(initres);string(endres);string(res)];
      if (input_param3==1) then
        answer_m=x_mdialog(title,txt,vis_value);
        if ~(answer_m==[]) then
          initres=evstr(answer_m(1));
          endres=evstr(answer_m(2));
          res=evstr(answer_m(3));
          initres=(round(initres/res))*res;
          endres=(round(endres/res))*res;
        else
          visualrec=0;
        end
      end
      if (visualrec==1) then
        if (endres < initres) then
          x_message(_("End resampling time must be greater than init resampling time"));
          error_f=1;
          input_param3=1;
        end
        if (res<0) then
          x_message(_("Resampling step must be greater than zero"));
          erro_f=1;
          input_param3=1;
        end
      end
    end
  end
  if (visualrec==1) then
    // time vector of resampling
    t=initres:res:endres;
    // SBP resampling by spline functions
    tSBP=SBPvect(:,2)';
    SBPv=SBPvect(:,1)';
    d=splin(tSBP,SBPv);
    SBPres=[(interp(t,tSBP,SBPv,d))',t'];
    // DBP resampling by spline functions
    tDBP=DBPvect(:,2)';
    DBPv=DBPvect(:,1)';
    d=splin(tDBP,DBPv);
    DBPres=[(interp(t,tDBP,DBPv,d))',t'];
    // writing of output files
    unix_res=unix("ls "+SBPo_name);
    if (unix_res==0) then
      unix_res=unix("rm "+SBPo_name);
    end
    unix_res=unix("ls "+DBPo_name);
    if (unix_res==0) then
      unix_res=unix("rm "+DBPo_name);
    end
    write(SBPo_name,SBPres,'(2(e14.6,3x))')
    write(DBPo_name,DBPres,'(2(e14.6,3x))')

    if ~((input_param==0)&(input_param2==0)&(input_param3==0)) then
      choise_vis=[_("Yes");_("No")];
      title_vis=[_("Visualisation of reconstructed features");
                 _("of cardiovascular signals")];
      VIS_sig=x_choose(choise_vis,title_vis);
      if (VIS_sig==1) then
        file_det=[SBPi_name;DBPi_name];
        file_rec=[SBPo_name;DBPo_name];
        VisualresamplingBP(file_det,file_rec,[]);
      end;
    end
  end;


endfunction
