// This file is part of the CardioVascular toolbox
// Copyright (C) - INRIA - Alessandro Monti, Claire Médigue, Michel Sorine
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
function []=ResamplingRawResp(listfile_in,listfile_out,res_par)

  input_param=0;
  visualrec=1;
  txt=[_("Respiratory signal file name")];
  title=['     '+_("Enter resampling parameters");..
         '     '+_("Reconstructed file names")];
  if ~((type(listfile_in)==10)&(size(listfile_in)==[1,1])) then
    input_param=1;
    RESPi_name='RawResp';
  else
    RESPi_name=listfile_in(1);
  end
  error_f=1;
  while ((error_f==1)&(visualrec==1)),
    error_f=0;
    i_value=[RESPi_name];
    if (input_param==1) then
      answer_m=x_mdialog(title,txt,i_value);
      if ~(answer_m==[]) then
        RESPi_name=answer_m(1);
      else
        visualrec=0;
      end
    end
    if (visualrec==1) then
      // test input file
      unix_res=unix("ls "+RESPi_name);
      if ~(unix_res==0) then
        x_message(msprintf(_(" ERROR: file %s does not exist"),"RESPi_name"))
        error_f=1;input_param=1;
      end
    end
  end
  //
  if (visualrec==1) then
    //   RECONSTRUCTION FILES
    //
    input_param2=0;
    if ~((type(listfile_out)==10)&(size(listfile_out)==[1,1])) then
      input_param2=1;
      RESPo_name='RawRespseries';
    else
      RESPo_name=listfile_out(1);
    end
    error_f=1;
    txt=[_("Respiratory  file name")];
    title=_("Enter RESAMPLED  files names");
    while ((error_f==1)&(visualrec==1)),
      error_f=0;
      o_value=[RESPo_name];
      if (input_param2==1) then
        answer_m=x_mdialog(title,txt,o_value);
        if ~(answer_m==[]) then
          RESPo_name=answer_m(1);
        else
          visualrec=0;
        end
      end
    end
  end
  input_param3=0;
  if (visualrec==1) then
    // read of RESP raw file
    RESPvect=read(RESPi_name,-1,2);
    RESPvect=[RESPvect(:,1),RESPvect(:,2)]; // time, valeur
    
    //   resampling PARAMETERS

    if ~((type(res_par)==1)&(size(res_par)==[3,1])) then
      input_param3=1;
      res=0.25;     
      initres=min(RESPvect(:,1));
      initres=(round(initres/res))*res+res;
      endres=max(RESPvect(:,1));     
      endres=(round(endres/res))*res-res;
    else
      initres=res_par(1);
      endres=res_par(2);
      res=res_par(3); // resampling step        
      initres=(round(initres/res))*res;
      endres=(round(endres/res))*res;
    end    
    error_f=1;
    txt=[_("Init resampling time");
         _("End resampling time");..
         _("Resampling step")];
    title=['  '+_("Resampling parameters")];
    while ((error_f==1)&(visualrec==1)),
      error_f=0;
      vis_value=[string(initres);string(endres);string(res)];
      if (input_param3==1) then
        answer_m=x_mdialog(title,txt,vis_value);
        if ~(answer_m==[]) then
          initres=evstr(answer_m(1));
          endres=evstr(answer_m(2));
          res=evstr(answer_m(3));
          initres=(round(initres/res))*res;
          endres=(round(endres/res))*res;
        else
          visualrec=0;
        end
      end
      if (visualrec==1) then
        if (endres < initres) then
          x_message(_("End resampling time must be greater than init resampling time"));
          error_f=1;
          input_param3=1;
        end
        if (res<0) then
          x_message(_("Resampling step must be greater than zero"));
          erro_f=1;
          input_param3=1;
        end
      end
    end
  end
  if (visualrec==1) then
    // time vector of resampling
    t=initres:res:endres;
    // RESP resampling by spline functions
    tRESP=RESPvect(:,1)';
    RESPv=RESPvect(:,2)';
    // oter les lignes debut correspondant au delay du filtre et commencant avec des 0
    deb=find(tRESP~=0);tRESP=tRESP(deb(1):$);RESPv=RESPv(deb(1):$);
    d=splin(tRESP,RESPv);
    RESPres=[(interp(t,tRESP,RESPv,d))',t'];
    // writing of output files
    unix_res=unix("ls "+RESPo_name);
    if (unix_res==0) then
      unix_res=unix("rm "+RESPo_name);
    end
    write(RESPo_name,RESPres,'(2(e14.6,3x))')
    if ~((input_param==0)&(input_param2==0)&(input_param3==0)) then
      choise_vis=[_("Yes");_("No")];
      title_vis=[_("Visualisation of reconstructed features");
                 _("of cardiovascular signals")];
      VIS_sig=x_choose(choise_vis,title_vis);
      if (VIS_sig==1) then
        file_det=[RESPi_name];
        file_rec=[RESPo_name];
        VisualresamplingRawResp(file_det,file_rec,[]);
      end;
    end
  end;


endfunction
