// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function TimeFrequencyTool(signal,freq_sampling,options)
  fs=freq_sampling;
  N=size(signal,'*')
  fig_ud=struct();
  fig_ud.signal=signal;
  fig_ud.fs=fs;
  fig_ud.freq_step=0.01
  fig_ud.fwl=max(5,round(N/40));
  fig_ud.twl=max(5,round(N/10));
  fig_ud.method="sp"
  fig_ud.draw_type="PseudoColor"
  fig_ud.cmap="jet"
  if argn(2)>=3 then
    F=fieldnames(options)
    for k=1:size(F,'*')
      if and(F(k)<>["freq_step","fwl","twl","method","draw_type","cmap"]) then
        error(msprint(_("%s: Wrong value for input argument #%d: Invalid field ""%s"".\n"),"TimeFrequencyTool",3,F(k)))
      end
      fig_ud(F(k))=options(F(k))
    end
    if and(fig_ud.method<>["sp","spwv","zam","bj"]) then
      error(msprint(_("%s: Wrong value for input argument #%d: Invalid ""%s"" field value.\n"),...
                    "TimeFrequencyTool",3,"method"))
    end
  end
  
  clf();fig=gcf()
  idraw=fig.immediate_drawing;
  fig.immediate_drawing="off";
 
  ax=gca();ax.axes_bounds=[0 0 1 2/3]
  xlabel(_("Time (sec)"));
  ylabel(_("Frequency (Hz)"));
  zlabel(_("Energy"));
  ax.tight_limits = "on";
 
  set(fig,"user_data",fig_ud)
  SA_SetCmap(fig_ud.cmap)
  SA_compute(fig);
  SA_draw(fig);
  fig.immediate_drawing = idraw;
  SA_gui(fig)
  delmenu(fig.figure_id,_("Control"))
  addmenu(fig.figure_id,_("Control"),[_("Hide");_("Show");_("Adjust")],list(2,"SA_ControlMenu"))
endfunction

function SA_gui(fig)
  fig_ud=fig.user_data;
  margin_x     = 5;      // Horizontal margin between each elements
  margin_y     = 5;      // Vertical margin between each elements
  label_h      = 20;
  list_w       = 180
  w1           = 60;
  w2           = 60;
  w3           = 100;
  frame_h      = fig.axes_size(2)/3;
  list_h       = frame_h-3*margin_y-label_h
  frame_w      = fig.axes_size(1)
  defaultfont  = "arial"; // Default Font

  
  frame=uicontrol( ...
      "parent"              , fig,...
      "style"               , "frame",...
      "units"               , "pixels",...
      "position"            , [0 0 frame_w frame_h],...
      "background"          , [1 1 1]*0.8, ...
      "visible"             , "on");
 
  y=frame_h-margin_y-label_h
  x=0;  
  //Method
  
  y=frame_h-margin_y-label_h
  x=0;  
   uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("TFR method"),...
      "units"               , "pixels",...
      "position"            , [x y list_w label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8);
   y=y-margin_y-list_h
   uicontrol( ...
      "parent"              , frame,...
      "style"               , "listbox",...
      "string"              , [_("Spectrogram");_("Smoothed Pseudo Wigner-Ville");_("Zao-Atlas-Marks");_("Born-Jordan")],...
      "units"               , "pixels",...
      "position"            , [x y list_w list_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "value"               , find(fig_ud.method==["sp","spwv","zam","bj"]),...
      "callback"            , "SA_SetMethod(gcbo.userdata(gcbo.value),gcbo.parent.parent)",...
      "userdata"            , ["sp";"spwv";"zam";"bj"],...
      "background"          , [1 1 1]*0.8);

   y=frame_h-margin_y-label_h
   x=x+list_w+margin_x;  
  //-----         Settings   --------------------
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("Settings"),...
      "units"               , "pixels",...
      "position"            , [x y w1+margin_x+w2 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8);
 
  //freq_step
  y=y-margin_y-label_h
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("freq_step"),...
      "units"               , "pixels",...
      "position"            , [x y w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8);
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "edit",...
      "string"              , string(fig_ud.freq_step),...
      "units"               , "pixels",...
      "position"            , [x+margin_x+w1 y w2 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "userdata"            , "freq_step", ...
      "callback"            , "SA_Settings()");
  //fwl
  y=y-margin_y-label_h
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("fwl"),...
      "units"               , "pixels",...
      "position"            , [x y w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8);
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "edit",...
      "string"              , string(fig_ud.fwl),...
      "units"               , "pixels",...
      "position"            , [x+margin_x+w1 y w2 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "userdata"            , "fwl", ...
      "callback"            , "SA_Settings()");
  //twl
  y=y-margin_y-label_h
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("twl"),...
      "units"               , "pixels",...
      "position"            , [x y w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8);
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "edit",...
      "string"              , string(fig_ud.twl),...
      "units"               , "pixels",...
      "position"            , [x+margin_x+w1 y w2 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8, ...
      "userdata"            , "twl", ...
      "callback"            , "SA_Settings()");

  
  //-----            --------------------

  y=frame_h-margin_y-label_h
  x=x+margin_x+w1+w2+margin_x;  
   uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("Display"),...
      "units"               , "pixels",...
      "position"            , [x y w1+margin_x+w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "fontweight"          , "bold", ...
      "horizontalalignment" , "center", ...
      "background"          , [1 1 1]*0.8);
 
  //Plot type
  y=y-margin_y-label_h
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("Type"),...
      "units"               , "pixels",...
      "position"            , [x y w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8);
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "popupmenu",...
      "string"              , ["PseudoColor";"3D"],...
      "units"               , "pixels",...
      "position"            , [x+margin_x+w1 y w3 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "value"               , find(fig_ud.draw_type==["PseudoColor";"3D"]),...
      "background"          , [1 1 1]*0.8, ...
      "callback"            , "SA_SetDisplay(gcbo.string(gcbo.value),gcbo.parent.parent)");
  //Color map
  y=y-margin_y-label_h
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "text",...
      "string"              , _("Cmap"),...
      "units"               , "pixels",...
      "position"            , [x y w1 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "background"          , [1 1 1]*0.8);
  uicontrol( ...
      "parent"              , frame,...
      "style"               , "popupmenu",...
      "string"              , ["Jet";"Gray"],...
      "units"               , "pixels",...
      "position"            , [x+margin_x+w1 y w3 label_h],....
      "fontunits"           , "points",...
      "fontsize"            , 12,...
      "horizontalalignment" , "left", ...
      "value"               , find(fig_ud.cmap==["jet";"gray"]),...
      "background"          , [1 1 1]*0.8, ...
      "callback"            , "SA_SetCmap(gcbo.string(gcbo.value),gcbo.parent.parent)");
endfunction

  
function SA_compute(fig)
  fig_ud=fig.user_data
  signal=fig_ud.signal
  fs=fig_ud.fs
  N=2^nextpow2(round(1/(fig_ud.freq_step)));
  n=size(signal,'*');
  timestep=round(n/N);
  T=[0:timestep:n-1 n];
  if modulo(fig_ud.twl,2)==0 then fig_ud.twl=fig_ud.twl+1,end
  if modulo(fig_ud.fwl,2)==0 then fig_ud.fwl=fig_ud.fwl+1,end
  Ns2=int(N/2)
  select fig_ud.method
  case "sp"
    H=window('hm',fig_ud.fwl)
    [SP,T,F] = Ctfrsp(hilbert(signal-mean(signal)),T,N,H);
    Ns2=N/2
  case "spwv"
    H=window('hm',fig_ud.fwl);
    G=window('hm',fig_ud.twl);
    [SP,T,F]=Ctfrspwv(hilbert(signal-mean(signal)),T,N,G,H);
    Ns2=N
  case "zam"
    H=window('hm',fig_ud.fwl);
    G=window('hm',fig_ud.twl);
    [SP,T,F]=Ctfrzam(hilbert(signal-mean(signal)),T,N,G,H);
    Ns2=N
  case "bj"
    H=window('hm',fig_ud.fwl);
    G=window('hm',fig_ud.twl);
    [SP,T,F]=Ctfrbj(hilbert(signal-mean(signal)),T,N,G,H);
    Ns2=N
  else
    error(_("Invalid method"))
  end
  
  fig_ud.T=T/fs
  fig_ud.F=F(1:Ns2)*fs
  fig_ud.SP=SP(1:Ns2,:)'
  set(fig,"user_data",fig_ud)
endfunction

function SA_draw(fig)
  scf(fig)
  fig_ud=fig.user_data
  SP=fig_ud.SP
  fig.immediate_drawing = "off"
  ax=gca();
  C=ax.children
  zb=ax.zoom_box
  select fig_ud.draw_type
  case "PseudoColor" then
    if size(C,'*')>0 then
      if C.type=="Grayplot" then
        data=tlist(["grayplotdata";"x";"y";"z"], fig_ud.T(:),fig_ud.F(:),SP)
        C.data=data
      else
        delete(C)
        grayplot(fig_ud.T,fig_ud.F,SP);
      end
    else
      grayplot(fig_ud.T,fig_ud.F,SP);
    end
    ax.view="2d"
    ax.cube_scaling="off";
  case "3D" then
    if size(C,'*')>0 then
      if C.type=="Plot3d" then
        data=tlist(["grayplotdata";"x";"y";"z"], fig_ud.T,fig_ud.F,SP)
        C.data=data
        ax.data_bounds(:,3)=[min(SP);max(SP)]
      else
        delete(C)
        plot3d1(fig_ud.T,fig_ud.F,SP);C=gce();
       end
    else
      plot3d1(fig_ud.T,fig_ud.F,SP);C=gce();
     end
    C.color_mode = -1;
    C.color_flag = 1;
    C.foreground = -1;

    if zb<>[] then
      zb(5:6)=[min(SP) max(SP)]
      ax.zoom_box=zb;
    end
    ax.cube_scaling="on";
    ax.view="3d"
  end
  fig.immediate_drawing = "on"
endfunction
function SA_SetMethod(m,fig)
  if argn(2)==1 then fig=gcf();end
  fig_ud=fig.user_data
  fig_ud.method=m
  set(fig,"user_data",fig_ud)
  SA_compute(fig)
  SA_draw(fig)
endfunction

function SA_SetDisplay(d,fig)
  if argn(2)==1 then fig=gcf();end
  fig_ud=fig.user_data
  select convstr(d)
  case "pseudocolor"  then 
    fig_ud.draw_type=d
    set(fig,"user_data",fig_ud)
    SA_draw(fig)
  case "3d"  then 
    fig_ud.draw_type=d
    set(fig,"user_data",fig_ud)
    SA_draw(fig)
  end
endfunction
function SA_SetCmap(d,fig)
  if argn(2)==1 then fig=gcf();end
  select convstr(d)
  case "jet" then  
    fig.color_map=jetcolormap(256);
  case "gray" then 
    cm=graycolormap(256)
    fig.color_map=cm($-10:-1:1,:); 
  end
endfunction


function SA_Settings()
  h=gcbo
  if execstr("v="+h.string,"errcatch")<>0 then
    h.ForegroundColor=[1 0 0]
    return
  else
    h.ForegroundColor=[0 0 0]
    key=h.userdata
    fig=h.parent.parent
    fig_ud=fig.user_data
    fig_ud(key)=v
    set(fig,"user_data",fig_ud)
    SA_compute(fig)
    SA_draw(fig)
  end
endfunction
function SA_ControlMenu(k,win)
  fig = get_figure_handle(win)
  C=fig.children
  c=C(C.type=="uicontrol")//the control panel
  g=C(C.type=="Axes")//the graph
  select k
  case 1 then //hide
    c.children.visible="off"
    c.visible='off"
    g.axes_bounds=[0 0 1 1]
  case 2 then //show
    c.visible='on"
    c.children.visible="on"
    g.axes_bounds=[0 0 1 2/3]
  case 3 then //adjust
    delete(c.children)
    delete(c)
    SA_gui(fig)
    draw(fig)
  end
endfunction
