// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// Copyright (C) - INRIA - Alessandro Monti, Claire Médigue, Michel Sorine
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function enablescrollingplot(fig)
  if argn(2)<2 then fig=gcf();end
  fig_ud=fig.user_data;
  if typeof(fig_ud)<>"st" then
    fig_ud=struct()
  end
  if ~isfield(fig_ud,"handlers") then
    fig_ud.handlers=[];
  end
  fig_ud.handlers=[fig_ud.handlers [fig.event_handler;fig.event_handler_enable]]

  set(fig,"user_data",fig_ud)
  fig.event_handler="scroll_event_handler";
  fig.event_handler_enable='off';
  addmenu(fig.figure_id,_("Scroll"),[_("On");_("Off")],list(2,"ScrollPlotMenu"))
endfunction
function ScrollPlotMenu(k,win)
  fig=get_figure_handle(win)
  select k
  case 1  then //On
    fig.event_handler="scroll_event_handler";
    fig.event_handler_enable='on';
  case 2  then //Off
    fig.event_handler_enable='off';
    fig_ud=fig.user_data
    if fig_ud.handlers==[] then
      fig.event_handler=""
    else
      fig.event_handler=fig_ud.handlers(1,$)
      fig.event_handler_enable=fig_ud.handlers(2,$)
      fig_ud.handlers(:,$)=[];
      set(fig,"user_data",fig_ud)
    end
  end
endfunction
function scroll_event_handler(win, x, y, ibut)
  if and(ibut<>[37:40 113]) then return,end
  fig=get_figure_handle(win)
  fig.event_handler_enable='off';
  ax=findSubwin(x,y,fig)
  if ax.zoom_box==[] then
    messagebox([_("Please select an area first");
                _("using the zoom menu")],"Scilab message","warning","ok","modal")
    
    fig.event_handler_enable='on';
    return
  end
  dx=ax.zoom_box(3)-ax.zoom_box(1)
  dy=ax.zoom_box(4)-ax.zoom_box(2)
  select ibut
  case 37 then //scroll left
    ax.zoom_box([1 3])=ax.zoom_box(1)-[dx 0];
  case 38 then //scroll up
    ax.zoom_box([2 4])=ax.zoom_box(4)+[0 dy];
  case 39 then //scroll right
    ax.zoom_box([1 3])=ax.zoom_box(3)+[0 dx];
  case 40 then //scroll down
    ax.zoom_box([2 4])=ax.zoom_box(2)-[dy 0];
  case 113 then //initial view
    ax.zoom_box=[];
  end
  fig.event_handler_enable='on';
endfunction
function ax=findSubwin(x,y,fig)
  sz=fig.axes_size;
  ax=[];
  C=fig.children;
  for k=1:size(C,'*')
    ax=C(k)
    if ax.type=="Axes" then
      bounds=ax.axes_bounds.*[sz sz]
      if x>=bounds(1)&x<(bounds(1)+bounds(3))&y>=bounds(2)&y<(bounds(2)+bounds(4)) then
        break
      end
    end
  end
endfunction
