function sig = filtbyfft(sig, fs, passband, tf)
//FILTBYFFT: filter by FFT
//
// FilteredSig = filtbyfft(Sig, Fs, Passband, TF)
//
// FilteredSig: filtered signal vector,
// Sig: signal vector,
// Fs: sampling frequency (in Hz),  
// Passband: passband interval (in Hz),
// TF: signal domain indicator, a string containg two characters, each being
//     of 't' or 'f'(time or frequency), the first one for SIG and the
//     second one for FilteredSig 
//
// Author: Qinghua Zhang
// Copyright 2009 INRIA

  if argn(2)<3  then
    error(msprintf(_("%s: Wrong number of input argument: At least %d expected.\n"),"filtbyfft",3))
  elseif argn(2)<4  then
    tf = 'tt';
  elseif and(tf<>["tt","tf","ft","ff"]) then
    error(msprintf(_("%s: Wrong value for input argument #%d: Must be in the set {%s}.\n"),"filtbyfft",4,["tt","tf","ft","ff"]))
  end

  if size(passband,"*")<>2 then
    error(msprintf(_("%s: Wrong size for input argument #%d: A %d elements array expected.\n"),"filtbyfft",3,2))
  end
  if passband(1)>=passband(2)|passband(1)<0 then
    error(msprintf(_("%s: Wrong values for input argument #%d: Elements must be in increasing order.\n"),"filtbyfft",3))
  end
  passband(2) = min(passband(2), 0.5*fs);

  N = length(sig);

  if part(tf,1)=='t' then
    sig = fft(sig);
  end

  lowicut = round(passband(1)*N/fs);
  lowmirror = N-lowicut+2;

  highicut =  round(passband(2)*N/fs);
  highicut = max(1, highicut); // Avoid highicut=0
  highmirror = N-highicut+2;

  sig([1:(lowicut-1) (lowmirror+1):N])=0;
  sig((highicut+1):(highmirror-1))=0;

  if part(tf,2)=='t' then
    //sig = ifft(sig, 'symmetric');
    sig = real(ifft(sig));
  end

endfunction
