// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [p,mu] = polyfit(x,y,n)

  if size(x,'*')<>size(y,'*') then
    error(mprintf(_("%s: Arguments #%d and #%d must have the same sizes.\n"),"polyfit",1,2))
  end

  x = x(:);
  y = y(:);

  if argn(1) > 2
    mu = [mean(x); st_deviation(x)];
    x = (x - mu(1))/mu(2);
  end

  // Construct Vandermonde matrix.
  nx=size(x,'*');
  V(1:nx,1:n+1) = 1;
  for j = n:-1:1
    V(:,j) = x.*V(:,j+1);
  end

  // Solve least squares problem.
  [Q,R] = qr(V,'e');
  p = R\(Q'*y);    // Same as p = V\y;
  if size(R,2) > size(R,1)
    warning(mprintf(_("%s:  Solution is not unique; degree >= number of data points.\n"),"polyfit"))
  elseif cond(R) < 1.0e-10
    if argn(2) > 2
      warning(mprintf(_("%s:  Solution is poorly conditionned. Remove repeated data points.\n"),"polyfit"))
    else
      warning(mprintf(_("%s:  Solution is poorly conditionned. Remove repeated data points or try centering and scaling.\n"),"polyfit"))
    end
  end
  r = y - V*p;
  p = p.';          // Polynomial coefficients are row vectors by convention.

endfunction

