// This file is part of the CardioVascular toolbox
// Copyright (C) 2012 - INRIA - Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [t,RR]=syntheticRRsignal(n)
//short term RR representative synthetic signal
//uses fmsin  of th  stftb toolbox
// t signal time instant in seconds
// RR signal in ms
  std=[0.5;4]; //standard deviations for the gaussian noise
  ampl_VLF=[0.5 0.6;0.8 0.9] //sinusoidal components amplitudes of VLF1
 
  RRmean=1000; //ms
  f_ech=4//hz
  t=0:0.25:600; //4hz sampling
  N=size(t,'*')
  

  //Very low frequency band signal (a 2 components signal)
  f1=0.005;//hz
  a1=ampl_VLF(n,1);//ms
  f2=0.03;//hz
  a2=ampl_VLF(n,1);//ms
  VLF=a1*sin(2*%pi*f1*t)+a2*sin(2*%pi*f2*t)
  
  //Low frequency band signal
  // frequency modulation
  Fmin=(0.09-0.01)/f_ech;Fmax=(0.09+0.01)/f_ech
  [LF,if_LF]=fmsin(N,Fmin,Fmax,150,0,(Fmin+Fmax)/2);
  // amplitude modulation
  IampLF=4;
  LF=IampLF*real(LF)';
  
  //High frequency band signal
  // frequency modulation
  Fmin=(0.25-0.01)/f_ech;Fmax=(0.25+0.01)/f_ech
  [HF,if_HF]=fmsin(N,Fmin,Fmax,80,0,(Fmin+Fmax)/2);
  // amplitude modulation
  k1=find(t<280);k1=k1($);
  k=find(305<=t&t<=340);k2=k(1);k3=k($);
  k4=find(t>=370);k4=k4(1);
  IampHF=[ones(1,k1-1),...
        linspace(1,0,k2-k1+1),...
        zeros(1,k3-k2-1),...
        linspace(0,10,k4-k3+1) ...
        10*ones(1,N-k4)];
  HF=Iamp.*real(HF)'
  
  //White gaussian additive noise
  WGN= grand(1,N,"nor",0,std(n));
  
  RR=RRmean+VLF+LF+HF+WGN
endfunction
